#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>
#include <string.h>
#include <malloc.h>
#include <assert.h>
#include "xpow.h"
#include "zlib.h"

                   /* Hold over from original BINET source */

#define DWELL_STRUCT  2
#define RADAR_STRUCT  1
#define RADAR_EOF    -1

                  /*  Function prototypes */

static char *getFieldName(int fnum);
static void file_open(FILE *(*in_ptr), char *in_name, char *file_type) ;

               /* Variables global to this file */

static char *xp_errmsg = "";  /* current error messages */
static char *xp_path = "";    /* path prefix for output files */
static FILE *xp_fp;           /* file pointer of current input file  */
static int xp_thfield;        /* field (XP_NONE, XP_NCP, ..) for threshold */
static float xp_thvalue;      /* set things below this to XP_BELOW_TH */
static float xp_nthfields;    /* # of threshold fields, <= 4 */
static int xp_bRunFlag = 1;   /* 0 => stop what you are doing */
static int xp_float = 0;      

       /* Accessor routines for the global variables */

void _XPSetThreshold(int tfield, float tvalue)
{
   xp_thfield = tfield;
   xp_thvalue = tvalue;
}

void _XPGetThresHold(int *thfield,float *thvalue)
{
   *thfield = xp_thfield;
   *thvalue = xp_thvalue;
}

void _XPSetRunState(int state) /* if xp_bRunFlag = 1, convert, else, don't */
{
   xp_bRunFlag = state;
}

int _XPGetRunState()
{
   return(xp_bRunFlag);
}

void _XPSetErrorMessage(char *m) /* Sets error message */
{
  xp_errmsg = m; 
}

void _XPSetPath(char *newpath)    /* Sets path prefix for output file names */
{
  xp_path = newpath;
}

void _XPSetFilePointer(FILE *fp)
{
  xp_fp = fp;
}

static unsigned int DBZ_INVF(float x)  /* reflectivities in DBZ */
{
   assert(x > -65.0);
   if (x == XP_BADVAL) return (unsigned int)0;
   if (x == XP_RFVAL)  return (unsigned int)1;
   if (x == XP_APFLAG) return (unsigned int)2;
   if (x == XP_APFLAG2)return (unsigned int)3;
   return((unsigned int)(64.0*(x+65.0)+.5 + 4));
}

static float DBZ_F(int x)              /* reflectivities in DBZ */
{
  if (x == 0) return (float)XP_BADVAL;
  if (x == 1) return (float)XP_RFVAL;
  if (x == 2) return (float)XP_APFLAG;
  if (x == 3) return (float)XP_APFLAG;
  return ((float)((x-4)/64.0 - 65.0));
}

static unsigned int PHDP_INVF(float x)  /* PhiDP in degrees */
{
   assert(x > -360.0);
   if (x == XP_BADVAL) return (unsigned int)0;
   if (x == XP_RFVAL)  return (unsigned int)1;
   if (x == XP_APFLAG) return (unsigned int)2;
   if (x == XP_APFLAG2)return (unsigned int)3;
   return((unsigned int)(96.0F*(x+360.0F)+.5F + 4));
}

static float PHDP_F(unsigned int x)         /*PhiDP in degrees */
{
  if (x == 0) return (float)XP_BADVAL;
  if (x == 1) return (float)XP_RFVAL;
  if (x == 2) return (float)XP_APFLAG;
  if (x == 3) return (float)XP_APFLAG;
  return ((float)((x-4)/96.0F - 360.0F));
}

static float GENERIC_F(int x)              /* generic float */
{
   int tmp = x;
   if (tmp < 0) tmp = 0;
   if (tmp >  65536) tmp =  65536;
   return((float)(tmp-32768)/100);    

}

static unsigned int GENERIC_INVF(float x)  /* generic float */
{  
   int tmp;

   if (x < -327.68F) x = -327.68F;
   if (x >  327.67F) x =  327.67F;
   tmp = (int)(100.0F*x);
   return((unsigned int)(tmp+32768));
}

static int _XPMapFloatToInt(float value,int field)
{
   /* Maps a float to an int */

  switch (field){
   case XP_HDBZ:
   case XP_VDBZ:
   case XP_VEL:
   case XP_NCP:
   case XP_SW:
      return(DBZ_INVF(value));
      break;
   case XP_PHDP:
      return(PHDP_INVF(value));
      break;
   case XP_GENERIC:
      return(GENERIC_INVF(value));
      break;
   default:
      return(GENERIC_INVF(value));
      break;
   }
}

static float _XPMapIntToFloat(int value,int field)
{
   switch (field){
   case XP_HDBZ:
   case XP_VDBZ:
   case XP_VEL:
   case XP_NCP:
   case XP_SW:
      return(DBZ_F(value));
      break;
   case XP_PHDP:
      return(PHDP_F(value));
      break;
   case XP_GENERIC:
      return(GENERIC_F(value));
      break;
   default:
      return(GENERIC_F(value));
      break;
   }
}

static char *getFieldName(int fnum)
{
   /* Maps the field number fnum to a character string */

   if (fnum == XP_VEL)   return("vel");
   if (fnum == XP_NCP)   return("ncp");
   if (fnum == XP_HDBM)  return("hdb");
   if (fnum == XP_SW)    return("sw");
   if (fnum == XP_HDBZ)  return("hdbz");
   if (fnum == XP_CDBZ)  return("cdbz");
   if (fnum == XP_ZDR)   return("zdr");
   if (fnum == XP_PHDP)   return("phdp");
   if (fnum == XP_VDBM1) return("vdb1");
   if (fnum == XP_COR)   return("cor");
   if (fnum == XP_VDBM2) return("vdb2");
   if (fnum == XP_VDBZ)  return("vdbz");
   if (fnum == XP_VDBM3) return("vdb3");
   return("err");
}

static int _XPReadAzimuth(FILE *fp,xp_sweep_header_t sweep,
                   xp_ray_header_t *ray, void *value)
{
   int i,tmp1;
   int vminor,vmajor;
   int c;
   float *fvalue = (float *)value;
   float *ivalue = (float *)value;

   /* Insurance so we don't try and read past the EOF */

   c = fgetc(fp);
   if (feof(fp)) return(XP_BADFILE);
   ungetc(c,fp);

   /* 
   ** Read the magic "number" (should be "XPOW") and the number
   ** of header lines.
   */

   for (i=0;i<=4-1;i++)
      ray->magic[i] = fgetc(fp);
   ray->magic[4] = '\0';
   fscanf(fp," %d",&ray->nheader_lines);

   /* Read the version number, and the encoding type. */

   fscanf(fp,"%d.%d",&vmajor,&vminor);
   ray->version = (float)(vmajor+vminor/10.0);
   fscanf(fp," %d",&ray->encoding);

   /* Time stamp */

   fscanf(fp," %d",&ray->year);
   fscanf(fp," %d",&ray->month);
   fscanf(fp," %d",&ray->day);
   fscanf(fp," %d",&ray->hour);
   fscanf(fp," %d",&ray->minute);
   fscanf(fp," %d",&tmp1);       /* seconds */
   ray->second = (float)tmp1;

   /* The rest of the story */

   fscanf(fp,"%d ",&ray->ngates);
   fscanf(fp,"%g ",&ray->azimuth);
   fscanf(fp,"%g ",&ray->elevation);
   fscanf(fp,"%d ",&ray->transition);
   fscanf(fp,"%d ",&ray->nhits);
   fscanf(fp,"%g ",&ray->tpulse);
   fscanf(fp,"%g ",&ray->rpulse);
   fscanf(fp,"%g ",&ray->prt);
   fscanf(fp,"%g ",&ray->latitude);
   fscanf(fp,"%g ",&ray->longitude);
   fscanf(fp,"%d ",&ray->scan_number);
   fscanf(fp,"%d ",&ray->volume_number);
   fscanf(fp,"%d ",&ray->scan_type);
   
   /* Field type: first numerical, then ASCII name */

   fscanf(fp,"%d ",&ray->field_type); 
   fscanf(fp,"%*[ \"]");                   /* read up (including) " */
   fscanf(fp,"%[^\"]",&ray->field_name);   /* read up (excluding) " */
   fscanf(fp,"%*[\"]");                    /* read final "  */
    
   /* 
   ** Now thresholds.  Hardcoded for now: 2 threshold with the
   ** first one hardcoded as XP_CDBZ = 0.0;
   */

   fscanf(fp,"%d ",&ray->nthfields);
   assert(ray->nthfields == 2);
   for (i=0;i<=(ray->nthfields)-1;i++)
      fscanf(fp,"%d %f ",&ray->thfield[i],&ray->thvalue[i]);
   assert(ray->thfield[0] == XP_CDBZ);

   /* Now the data */
   
   assert((ray->ngates)<= XP_MAXGATES);
   for (i=0;i<=(ray->ngates-1);i++){
      if (xp_float)
         fscanf(fp,"%f",&fvalue[i]);
      else
         fscanf(fp,"%d",&ivalue[i]);
   }

   /* Read up to and including the newline */
   
   while((c= fgetc(fp)) != '\n'){
      if (feof(fp)) break;
   }
   return(XP_SUCCESS);
}

static void _XPWriteAzimuth(FILE *fp,xp_sweep_header_t sweep,xp_ray_header_t ray,
                 void *value)
{
   int i,tmp1,tmp2;
   float *fvalue = (float *)value;
   int *ivalue = (int *)value;

   /* 
   ** Write the magic "number" (should be "XPOW") and the number
   ** of header lines.
   */

   fprintf(fp,"%s ",ray.magic);
   fprintf(fp,"%d ",ray.nheader_lines);

   /* Write the version number, and the encoding type. */

   fprintf(fp,"%3.1f %d ",ray.version,ray.encoding);

   /* Time stamp */

   fprintf(fp,"%4d ",ray.year);
   fprintf(fp,"%02d ",ray.month);
   fprintf(fp,"%02d ",ray.day);
   fprintf(fp,"%02d ",ray.hour);
   fprintf(fp,"%02d ",ray.minute);
   
   tmp1 = (int)ray.second;
   tmp2 = (int)((ray.second - tmp1)*1000);  /* ignore subseconds */
   fprintf(fp,"%02d ",tmp1);                /* seconds */
 
   /* The rest of the story */

   fprintf(fp,"%d ",ray.ngates);
   fprintf(fp,"%8.4f ",ray.azimuth);
   fprintf(fp,"%8.4f ",ray.elevation);
   fprintf(fp,"%d ",ray.transition);
   fprintf(fp,"%d ",ray.nhits);
   fprintf(fp,"%g ",ray.tpulse);
   fprintf(fp,"%g ",ray.rpulse);
   fprintf(fp,"%g ",ray.prt);
   fprintf(fp,"%g ",ray.latitude);
   fprintf(fp,"%g ",ray.longitude);
   fprintf(fp,"%d ",ray.scan_number);
   fprintf(fp,"%d ",ray.volume_number);
   fprintf(fp,"%d ",ray.scan_type);
   
   /* Field type: first numerical, then ASCII name */

   fprintf(fp,"%d ",ray.field_type);
   fprintf(fp,"\"%s\"",ray.field_name);     

   /* Thresholds */

   fprintf(fp," %d ",ray.nthfields);
   for (i=0;i<=ray.nthfields-1;i++)
      fprintf(fp,"%d %4.1f ",ray.thfield[i],ray.thvalue[i]);
   fprintf(fp,"\n");   

   /* Now the data */

   for (i=0;i<=(ray.ngates-1);i++){
      if (xp_float) 
         fprintf(fp,"%10.5f ",fvalue[i]);
      else  
         fprintf(fp,"%d ",ivalue[i]);
   }
   fprintf(fp,"\n");     
}

static char *_XPGetScanType(xp_ray_header_t ray)
{
   /* Returns and ASCII string of the scan type */

   static char scan_type[10];     /* hardcoded */

   if(ray.scan_type == PPI)
      sprintf(scan_type, "ppi");
   else if(ray.scan_type == SUR) 
      sprintf(scan_type, "sur");
   else if(ray.scan_type == RHI) 
      sprintf(scan_type, "rhi");
   else 
      sprintf(scan_type, "unkn");
  return(&scan_type[0]);
}

static char *_XPMakeFileName(xp_ray_header_t ray)
{
   /* Builds a file name from the contents of "ray" */

   char *str1;
   char *str2;
   static char buffer[4096];

   str1 = _XPGetScanType(ray);   /* "PPI", "RHI", etc. */
   str2 = &(ray.field_name[0]); /* "hdbz", "vel", etc */

   if (xp_path == NULL) {


     sprintf(buffer,"%s_%04d%02d%02d_%02d%02d%02d.%s",
             str1,ray.year,ray.month,ray.day,ray.hour,ray.minute,
             (int)ray.second,str2);
   }
   else {

     /*                       y   m   d    h   m    */

     sprintf(buffer,"%s%s_%04d%02d%02d_%02d%02d%02d.%s",
             xp_path,str1,ray.year,ray.month,ray.day,ray.hour,ray.minute,
             (int)ray.second,str2);
   }
   return(&buffer[0]);
}


int XPClose(XP_FILE *xpfp)
{
   free(xpfp->dest);
   free(xpfp->buff);
   return(fclose(xpfp->fp));
}

XP_FILE *XPOpen(char *file,char * mode)
{
   /* Opens the XPOW file */

   static XP_FILE xpfp;
   static FILE *fp;
   int nread;
   char str[1024];               /* hardcoded */
  
   if ((fp=fopen(file,"rb")) == NULL)
      return(NULL);
   setvbuf(fp,NULL,_IOFBF,(size_t)XP_BUFFSIZE);
   xpfp.fp = fp;
   xpfp.bp = 0;

   nread = fread(str,1,36,xpfp.fp);
   sscanf(&str[6],"%6d%6d%6d%6d",
         &xpfp.dlen,&xpfp.slen,&xpfp.tnrays,&xpfp.headsize);

   /* 
    ** Allocate the work arrays: "slen" is the size in bytes of the data
   ** that we will compress. Each gate is coded in 2 bytes, and the #define
   ** XP_CODED_HEADSIZE is the size of the header.  See the include file for
   ** more information.  The variable "dlen" is the size of the buffer
   ** that will hold the compressed data.  The ZLIB documentation says
   ** it has to be 0.1% (I use 5) + 12 bytes larger than "slen".
   */

   xpfp.buff = xpfp.dest = NULL;
   if ((xpfp.buff = malloc(xpfp.slen)) == NULL) {
      fclose(xpfp.fp);
      return(NULL);               /* more work */
   } 

   if ((xpfp.dest = malloc(xpfp.dlen)) == NULL) {
      free(xpfp.buff);
      fclose(xpfp.fp);
      return(NULL);               /* more work */
   }
   return(&xpfp);
}

int XPReadSweepHeader(XP_FILE* xpfp, xp_sweep_header_t *sweep)
{
  unsigned long slen;
  int retval,nread;

  /* Read the whole compressed file into "dest" in xpfp */

  nread = fread(xpfp->dest,1,xpfp->dlen,xpfp->fp);
  assert(nread == xpfp->dlen);
  
  /* Uncompress the data.  The output goes to "buff" in xpfp */
  
  slen = xpfp->slen;
  /*                  <---  dest --->  <----    src    ----> */  
  retval = uncompress(xpfp->buff,&slen,xpfp->dest,xpfp->dlen);
  assert(retval == Z_OK);

  /* 
  ** Unpack the first ray header information. This becomes the sweep
  ** header. Reset the pointer.
  */

  retval = _XPUnPackHeader(sweep,xpfp->buff);
  xpfp->bp = 0;
  return(XP_SUCCESS);
}

int XPReadAzimuth(XP_FILE *xpfp,xp_sweep_header_t sweep, xp_ray_header_t *ray,
                     float *value)
{
   int retval,i,itmp;

   /* Next header */

   retval = _XPUnPackHeader(ray,&(xpfp->buff[xpfp->bp]));
   xpfp->bp = xpfp->bp + retval;

   /* The values */
  
   for (i=0;i<=ray->ngates-1;i++){
      itmp = xpfp->buff[xpfp->bp++] & 0377;             /* byte 0 */
      itmp |= ((xpfp->buff[xpfp->bp++] & 0377) << 8);   /* byte 1 */
      value[i] = _XPMapIntToFloat(itmp,ray->field_type);
   }
   return(XP_SUCCESS);
}

