#ifndef _LIB_XPOW_INCLUDED

#include <stdio.h>
#include <stdlib.h>

            /* Scanning strategies.  This comes from the BINET software */

#define PPI        1     
#define SUR        2
#define RHI        3
#define TRANSITION 7

/* 
** Offset of variables in products array.  This was constructed by
** examining the Binet software. The original software has some fields 
** reversed.  
**
** A.K. June, 2001.
**
*/

#define XP_VEL      0    /* Doppler Velocity (m/s)                 */
#define XP_HDBM     1    /* Horizontal Power at test coupler (dBm) */
#define XP_NCP      2    /* Noise ratio NCP                        */
#define XP_SW       3    /* Spectral Width (m/s)                   */
#define XP_HDBZ     4    /* Horizontal Reflectivity (dBZ)          */
#define XP_CDBZ     5    /* Coherent Reflectivity (dBZ)            */
#define XP_ZDR      6    /* ZDR (dB)                               */
#define XP_PHDP     7    /* Differiential Phase (deg)              */
#define XP_VDBM1    8    /* Vertical Power (dBm) see note 1 above  */
#define XP_COR      9    /* Cross Correlation                      */
#define XP_VDBM2    10   /* Vertical Power (dBm) see note 1 above  */
#define XP_VDBZ     11   /* Vertical Reflectivity (dBZ)            */
#define XP_VDBM3    12   /* Vertical Power (dBm) see note 1 above  */
#define XP_EMP1     13   /* Empty Field 1                          */
#define XP_EMP2     14   /* Empty Field 2                          */
#define XP_EMP3     15   /* Empty Field 3                          */
#define XP_GENERIC  16   /* Any field                              */

               /* Return values from XP routines */

#define XP_TRUE          1
#define XP_FALSE         0
#define XP_SUCCESS       1   /* successful execution or routine */
#define XP_FAILURE       0   /* failed for some reason */    
#define XP_STOPPED      -1   /* User stopped processing */
#define XP_NOMEM        -2   /* failed: malloc for work arrays failed */  
#define XP_BADFILE      -3   /* disk error occured */
#define XP_EODATA       -4   /* end of data in file */
#define XP_BADDATA      -5   /* unexpected data in file */

#define XP_BADVAL       320  /* => clear-air */ 
#define XP_RFVAL        (XP_BADVAL-1)
#define XP_APFLAG       (XP_BADVAL-2)
#define XP_APFLAG2      (XP_BADVAL-3)

               /* General #defines */

#define XP_NOSOCKETS     1    /* or XP_SOCKETS, but only "NO" for now */ 
#define XP_BINARY_ENC    0    /* packed binary encoding */
#define XP_ASCII_ENC     1    /* packed ASCII encoding */
#define XP_SIMPLE_ENC    2    /* simple ASCII encoding */

#define XP_COMMENT_CHAR '#'   /* start of comment lines in index file */
#define XP_VERSION    (1.1)   /* version of libray and format */  
#define XP_NONE         -1
#define XP_CLEAR_AIR   320    /* signals a value below clear air threshold */
#define XP_BUFFSIZE   4096    /* buffer size for setvbuf calls */

   /* Maximum number of gates, from BINET's globals.h file */
   /* A.K. Sunday, July 08, 2001 */
 
#define XP_MAXGATES   1024    /* used by some internal buffers */
#define XP_MAXSCANS   1024    /* index files must be smaller than this */
#define XP_MAXLINE     255    /* Maximum line length in XPB index files */

   /* The next macro returns byte "n" in "x" */

#define XP_GETBYTE(n,x) (((x) >> (8*(n))) & 0377)

/* The ray header.  
**
** Note 1: The layout here is the same as in the file headers.
**         Thus, on disk, a ray starts with the 10-byte magic string,
**         then follows, nheader_lines, etc.
**
** Note 2: In the conversion, the fractional part of the ray, second
**         is discarded, so we could have used, an int variable. However
**         in the future we may use the fractional part.  Also, I am not sure
**         if leap seconds are handled.
**
** Note 3: The meaning of some fields in the BINET format are not clear, but
**         they may prove useful later, so we keep them.
*/

typedef struct {
   char   magic[10];       /* Magic string. First 4: "XPOW", rest \0 */
   int    nheader_lines;   /* Number of lines in this header */
   float  version;         /* Version of the XPOW format */
   int    encoding;        /* XP_BINARY_ENC, XP_ASCII_ENC, XP_SIMPLE_ENC */

   int    year;            /* 2000, 2001, ... */
   int    month;           /* Jan = 1, Dec = 12 */
   int    day;             /* 0...31 */
   int    hour;            /* 0...23 */
   int    minute;          /* 0...59 */
   float  second;          /* 0...59.0 (Note 2) */

   int    nrays;           /* Number of rays in this sweep */
   int    ray_number;      /* 0,1,..nrays-1  */
   int    ngates;          /* Number of gates in the dwell/ray */
   float  azimuth;         /* Azimuth (degrees) for this dwell/ray */
   float  elevation;       /* Elevation angle (degrees) for this dwell/ray */
   int    transition;      /* 1 => in transition from PPI->RHI, etc */
                           /* 0 = in middle of PPI, RHI */

   int    nhits;           /* Number of hits */
   float  tpulse;          /* Transmitter pulse width (s) */
   float  rpulse;          /* Receiver pulse width (s) */
   float  prt;             /* Pulse repetition time (s) */
   
   float  latitude;        /* Radar latitute (from on-board GPS) */
   float  longitude;       /* Radar longitude (from on-board GPS) */
      
   int    scan_number;     /* Meaning unclear (Note 3) */
   int    volume_number;   /* Meaning unclear  (Note 3)*/
   int    field_type;      /* Field code, i.e., XP_NCP (see above) */
   char   field_name[10];  /* ASCII equivalent, i.e., "hdbz", "vel" */

   int    nthfields;       /* Number of fields used for thresholding <= 4 */
   int    thfield[4];      /* Threshold field: XP_NCP, XP_HDBZ, XP_NONE */
   float  thvalue[4];      /* Value that was used for threshold */

   int    scan_type;       /* One of: RHI, PPI, ,,, see #defines above */

   long   error_number;    /* For future expansion */
   long   warning_number;  /* For future expansion */
   int    reserved[16];    /* For future expansion */
   void   *ext;            /* For future expansion */

} xp_ray_header_t,xp_sweep_header_t;

 
typedef struct {
  FILE *fp;                 /* Current open file */
  unsigned long bp;      
  unsigned char *dest;
  unsigned char *buff;
  int dlen,slen;
  int headsize;
  int tnrays,tngates;
} XP_FILE;

                    /* Funtion prototypes */

#if defined(_WIN32) && defined(__cplusplus)
extern "C" {
#endif
               /*  Internal Start*/

void _XPSetRunState(int state);
int _XPGetRunState(void);
void _XPSetPath(char *newpath);
long _XPGetFilePos(void);
void _XPSetFilePointer(FILE *fp);
void _XPSetErrorMessage(char *m);
void _XPSetThreshold(int tfield, float tvalue);
void _XPGetThresHold(int *thfield,float *thvalue);
int _XPGetCodedHeaderSize(void);
int _XPPackHeader(xp_ray_header_t *head, unsigned char *buff);
int _XPUnPackHeader(xp_ray_header_t *head, unsigned char *buff);

int _XPPass1(char *infile, int *farr, int num_inputfields);
int _XPPass2(char *infile);
int _XPPass3(char *infile);
   
               /* Internal End */

XP_FILE *XPOpen(char *file,char * mode);
int XPReadSweepHeader(XP_FILE* xpfp, xp_sweep_header_t *sweep);
int XPReadAzimuth(XP_FILE *xpfp,xp_sweep_header_t sweep, xp_ray_header_t *ray,
                  float *value);
int XPClose(XP_FILE *xpfp);

                     /* test for data */

#define XP_ISDATA(x) (((x) >= (float)(XP_BADVAL-3)) ? XP_FALSE : XP_TRUE) 


#if defined(_WIN32) && defined(__cplusplus)
}
#endif

#define _LIB_XPOW_INCLUDED
#endif













