/*
** XPOW file read
** Process the CAMEX 4 data file.
*/
#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
#include "xpow.h"

#define  MAX_LENGTH  1024       		/* maximum file name length */

int main(void)
{
  XP_FILE *xpfp;                		/* similar to FILE, declared in "xpow.h" */
  xp_ray_header_t   ray;        		/* declared in "xpow.h" */
  xp_sweep_header_t sweep;      		/* declared in "xpow.h" */
  FILE   *fpp, *fphd, *fpvd, *fpph;
  FILE	 *fpsw, *fpvel, *fout;
  char   xpowfile[MAX_LENGTH];
  char   xpowdir[900],filer[900];
  char   dir_hd[900],dir_vd[900];
  char   dir_ph[900],dir_sw[900],dir_vel[900];
  int    i,j,iaz,Nrg,Nrays,ray_gates[500];
  float  Zh[361][500],Zv[361][500],vel[361][500];
  float  sw[361][500],Phd[361][500];
  float  vv1,vv2,vv3,vv4,vv5,vv6,elev,gatesize,ray_azimuth[500];
  float  value[5000];            		/* dBZ, KDP, etc. values */

  /* Note: value 320 in the data corresponds to reflectivity below 0 dBZ */

  sprintf(dir_hd,"Data/list_ppi2_hdbz");	/* list of Zh files (in dBZ) */
  sprintf(dir_vd,"Data/list_ppi2_vdbz");	/* list of Zv files (in dBZ) */
  sprintf(dir_ph,"Data/list_ppi2_phdp");	/* list of PhiDP files (in deg) */
  sprintf(dir_sw,"Data/list_ppi2_sw");		/* list of Spectral Width files (m/s) */
  sprintf(dir_vel,"Data/list_ppi2_vel");	/* list of Velocity files (m/s) */

  fphd=fopen(dir_hd,"r"); fpvd=fopen(dir_vd,"r"); fpph=fopen(dir_ph,"r");
  fpsw=fopen(dir_sw,"r"); fpvel=fopen(dir_vel,"r");

  while(!feof(fphd))
      {
    /* Open XPOW HDBZ file */
      fscanf(fphd,"%s",filer);
      sprintf(xpowfile,"Data/%s\0",filer);
      if ((xpfp=XPOpen(xpowfile,"rb"))==NULL){
        fprintf(stderr,"ERROR(2): could not list file %s\n",xpowfile);
        exit(-1);
        }
      XPReadSweepHeader(xpfp,&sweep);
      gatesize=sweep.rpulse*150000;                            /* gate size in km */
      Nrays=sweep.nrays;                                       /* number of rays in a sweep */
      elev=sweep.elevation;                                    /* sweep elevation */
      printf("%d %f %f\n",Nrays,gatesize,elev);

      for (i=0;i<sweep.nrays;i++) 				/* read the sector scan */
        {
        XPReadAzimuth(xpfp,sweep,&ray,value);
        ray_azimuth[i]=ray.azimuth;                            /* azimuth of each ray */
        ray_gates[i]=ray.ngates;                               /* number of bins in a ray */
        for(j=0;j<ray.ngates;j++)
              Zh[i][j]=value[j];
        }
      XPClose(xpfp);

    /* Open XPOW VDBZ file */
      fscanf(fpvd,"%s",filer);
      sprintf(xpowfile,"Data/%s\0",filer);
      if ((xpfp=XPOpen(xpowfile,"rb"))==NULL){
        fprintf(stderr,"ERROR(2): could not list file %s\n",xpowfile);
        exit(-1);
        }
      XPReadSweepHeader(xpfp,&sweep);
      for (i=0;i<sweep.nrays;i++) 				/* read the sector scan */
        {
        XPReadAzimuth(xpfp,sweep,&ray,value);
        for(j=0;j<ray.ngates;j++)
              Zv[i][j]=value[j];
        }
      XPClose(xpfp);
  
    /* Open XPOW PHDP file */
      fscanf(fpph,"%s",filer);
      sprintf(xpowfile,"Data/%s\0",filer);
      if ((xpfp=XPOpen(xpowfile,"rb"))==NULL){
        fprintf(stderr,"ERROR(2): could not list file %s\n",xpowfile);
        exit(-1);
        }
      XPReadSweepHeader(xpfp,&sweep);
      for (i=0;i<sweep.nrays;i++) 				/* read the sector scan */
        {
        XPReadAzimuth(xpfp,sweep,&ray,value);
        for(j=0;j<ray.ngates;j++)
              Phd[i][j]=value[j];
        }
      XPClose(xpfp);
    
    /* Open XPOW SW file */
      fscanf(fpsw,"%s",filer);
      sprintf(xpowfile,"Data/%s\0",filer);
      if ((xpfp=XPOpen(xpowfile,"rb"))==NULL){
        fprintf(stderr,"ERROR(2): could not list file %s\n",xpowfile);
        exit(-1);
        }
      XPReadSweepHeader(xpfp,&sweep);
      for (i=0;i<sweep.nrays;i++) 				/* read the sector scan */
        {
        XPReadAzimuth(xpfp,sweep,&ray,value);
        for(j=0;j<ray.ngates;j++)
              sw[i][j]=value[j];
        }
      XPClose(xpfp);

    /* Open XPOW VEL file */
      fscanf(fpvel,"%s",filer);
      sprintf(xpowfile,"Data/%s\0",filer);
      if ((xpfp=XPOpen(xpowfile,"rb"))==NULL){
        fprintf(stderr,"ERROR(2): could not list file %s\n",xpowfile);
        exit(-1);
        }
      XPReadSweepHeader(xpfp,&sweep);
      for (i=0;i<sweep.nrays;i++) 				/* read the sector scan */
        {
        XPReadAzimuth(xpfp,sweep,&ray,value);
        for(j=0;j<ray.ngates;j++)
	  {
              vel[i][j]=value[j];
	      if(Zh[i][j] > 40.0 && Zh[i][j] < 70.0)
		printf("%f %f %f %f %f\n",Zh[i][j],Zv[i][j],Phd[i][j],sw[i][j],vel[i][j]);
	  }
        }
      XPClose(xpfp);
      }
  
  fclose(fphd); fclose(fpvd); fclose(fpph); fclose(fpsw); fclose(fpvel);

exit(0);
}
